Time of Flight demo
===================

This demo application consists of

1. GUI for programming devices and parsing UART samples
2. Precompiled embedded executable

The demo uses two CC2640R2 Launchpads, where one device is *Master* and sends
ToF *ping* requests to the *Slave* which responds.

The precompiled executable boots up into *Slave* mode, where it operates as
follows:
* Set frequency
* Listen for *ping* until timeout
* If timeout reached, set new frequency and repeat
* If ping received, repeat listening on same frequency

If the device is told to enter *Master* mode, either by clicking **BTN-1** on
the Launchpad, or by receiving 'm' over UART it operates as follows:
* Set frequecy
* Send ping 128 times
* Store timestamp delta of valid responses
* Transmit samples over UART
* Repeat with new frequency


To open the README again, click Help -> README.md.


Requirements
------------

2x CC2640R2 Launchpad


Setting up
-----------

### 1. Program the slave device

Connect *one* of the CC2640R2 Launchpads to your computer. Then download the demo
firmware onto the device by clicking File -> Program Device...

<img src="resources_readme/programdevice.png"/>

After this is completed, disconnect this launchpad and power it from some other
source than the computer you are running the GUI on.

Once it powers up it becomes a slave.

### 2. Program the master device

Connect the other CC2640R2 Launchpad to your computer and program it like above.
Do not disconnect afterwards.

### 3. Configure the serial port.

Find the COM port number corresponding to the Application/User UART port
of the XDS110 debugger.

<img src="resources_readme/configserial_devmgr.png"/>

To connect the GUI to the master device, select Options -> Serial port.

<img src="resources_readme/configserial_menu.png"/>

Configure this to use the COM port you found, and make sure the baudrate is
460800bps.

<img src="resources_readme/configserial_config.png"/>

Click "Configure". Now the GUI should open the COM port and send the command
that configures the device as a *Master*.


Using the GUI
--------------

The GUI is made up of several parts, and an overview is presented below:

<img src="resources_readme/gui_overview.png"/>

### 1. Raw ToF samples
<img src="resources_readme/gui_rawtof.png"/>

This view displays raw ToF samples per frequency. Each "dot" represents a burst
of 128 pings, which have been reduced to one value.

1. The grey line represents the percentage of valid responses received during a burst.
2. The blue line represents the geometrical mean of the valid frames in a burst
3. The red line represents samples that have been filtered with a user-filter.
4. The average value is the average over 40 samples, and is used for calibration.

The unit in these graphs is `ticks` and not meters.

### 2. Calibration view

The text box is a JSON representation of the average TOF value over 40 bursts on
each frequency. It is used to be able to combine the measurements from several
frequencies. The calibration values are used as an offset.

Here you can also **RE-CALIBRATE** which will take the average of the latest
40 samples from the Raw view above.

You can also **SAVE CALIBRATION** which will store the values in
`calibration.json` which is a file local to the project.

### 3. Distance meter

This shows the average of the last 8 samples in the main combined graph.

### 4. Main graph

This graph shows the calibrated mean from each frequency. The input is the
*filtered* value (red line) from the Raw view.

### 5. Clear/Download

The **CLEAR SAMPLES** button will clear the record of samples stored in RAM and
which are displayed on the screen.

The **DOWNLOAD SAMPLES** button will allow you to save all the raw and filtered
samples since the GUI started or since samples were last cleared.

### 6. Burst filter

This is an optional function that will be applied to the 128 samples in a burst,
if enabled.

The output of this is stored as `frame.filtered_burst_tof`.

If this filter is not enabled, then the output value is set equal to the average
of the valid samples.

### 7. Time filter

This is an optional function that is applied to the frame as a whole. The
output is stored as `frame.filtered_tof`, and this value is used by the filtered
lines and the main graph.

If not enabled, the output values is set equal to `frame.filtered_burst_tof` from
the filter function above.
